/* game.c */
#include "global.h"
#include <stdlib.h>
#include "game.h"
#include "draw.h"
#include "sound.h"
#include "input.h"
#include "logic.h"

/*     */
#include <mmsystem.h>
#pragma comment(lib,"winmm.lib")

/*     FPS */
#define FPS_BUFF                          5

/*   */
#define GAME_STATE_STARTING               1
#define GAME_STATE_RUN                    2
#define GAME_STATE_EXIT                   3

/*   */
static int game_state;          /*    */
static int level;               /*    */
static DWORD dwLastTick;        /*      */

static int fpsBuff[FPS_BUFF];   /*    FPS */
static int pFps;                /*     */
int fps;                        /*  FPS */

static int speedBuff[FPS_BUFF];
static int pSpeed;
int speed;

/*   */
int err;

/*   */
static int GameStarting(void)
{
	/*   */
	if (CreateLevel(level))
		return 1;
	/*     */
	if (CreateScene())
		return 1;

	return 0;
}

/*   */
static void GameRun(void)
{
	DWORD dwTempTime = timeGetTime();

	/*    */
	Input();
	/*    */
	MoveScene();
	/*   */
	Sound();
	/*   */
	if (DrawScene())
		err = 1;

	speedBuff[pSpeed] = timeGetTime() - dwTempTime;
	if (++pSpeed >= FPS_BUFF) {
		int i,sum;
		for (i = sum = 0; i < FPS_BUFF; i++)
			sum += speedBuff[i];
		speed = sum / FPS_BUFF;
		pSpeed = 0;
	}
}

/*    */
static void GameExit(void)
{
	/*   */
	ReleaseScene();
}

/*    */
int MainGame(void)
{
	INT nSleepTime;

	/*   ESC */
	if (GetAsyncKeyState(VK_ESCAPE) & 0x8000)
		game_state = GAME_STATE_EXIT;

	/*     */
	switch (game_state) {
		/*   */
		case GAME_STATE_STARTING:
			/*     */
			level = 1;
			if (GameStarting())
				game_state = GAME_STATE_EXIT;
			else
				game_state = GAME_STATE_RUN;
			return 0;

		/*   */
		case GAME_STATE_RUN:
			GameRun();
			break;

		/*    */
		case GAME_STATE_EXIT:
			GameExit();
			PostMessage(ghWndMain,WM_CLOSE,0,0);
			return 0;
	}

	/*     FPS */
	nSleepTime = DELAY_FPS - (timeGetTime() - dwLastTick);
	if (nSleepTime > 1) Sleep(nSleepTime-1);
	while ((timeGetTime() - dwLastTick) < DELAY_FPS);

	/*  FPS */
	if (bFps) {
		/*    */
		fpsBuff[pFps] = timeGetTime() - dwLastTick;
		/*   ,  FPS */
		if (++pFps >= FPS_BUFF) {
			int i,sum,delay;
			/*    */
			for (i = sum = 0; i < FPS_BUFF; i++)
				sum += fpsBuff[i];
			/*    */
			delay = sum / FPS_BUFF;
			/*  FPS */
			fps = 1000 / delay;
			/*    */
			pFps = 0;
		}
	}

	/*     */
	dwLastTick = timeGetTime();

	return 0;
}

/*   */
int InitGame(void)
{
	/*   */
	ShowCursor(FALSE);

	/*    */
	if (InitDraw())
		return 1;
	/*    */
	if (InitSound())
		return 1;
	/*    */
	if (InitInput())
		return 1;

	/*   */
	game_state = GAME_STATE_STARTING;

	/*   */
	timeBeginPeriod(1);

	/*     */
	srand(timeGetTime());

	return 0;
}

/*   */
void ShutdownGame(void)
{
	/*   */
	timeEndPeriod(1);

	/*    */
	ReleaseInput();
	/*    */
	ReleaseSound();
	/*    */
	ReleaseDraw();

	/*   */
	ShowCursor(TRUE);
}
