/* input.c */

/*  guids */
#define INITGUID
/*  8-  DirectInput */
#define DIRECTINPUT_VERSION     0x0800

/*   */
#include "global.h"
#include <dinput.h>
#include "input.h"

/*   DINPUT */
#pragma comment(lib,"dinput8.lib")

/*    */
#define DIKEYDOWN(data,n)      ((data)[(n)] & 0x80)

/*   */
INPUT input;

/*   */
static LPDIRECTINPUT8 lpdi;                /*   DirectInput */
static LPDIRECTINPUTDEVICE8 lpdiKey;       /*   */
static LPDIRECTINPUTDEVICE8 lpdiMouse;     /*   */

/*      */
static struct {
	int q;
	int w;
	int e;
} flags;

/*    */
int Input(void)
{
	UCHAR keyState[256];
	DIMOUSESTATE mouseState;

	/*    */
	if (FAILED(IDirectInputDevice8_GetDeviceState(lpdiKey,
		sizeof(keyState),keyState)))
		return 1;
	/*    */
	if (FAILED(IDirectInputDevice8_GetDeviceState(lpdiMouse,
		sizeof(DIMOUSESTATE),&mouseState)))
		return 1;

	/*    */
	ZeroMemory(&input,sizeof(input));
	if (DIKEYDOWN(keyState,DIK_K)) input.K = 1;
	if (DIKEYDOWN(keyState,DIK_L)) input.L = 1;

//	if (DIKEYDOWN(keyState,DIK_Q)) input.down_Q = 1;
//	if (DIKEYDOWN(keyState,DIK_W)) input.down_W = 1;

	if (DIKEYDOWN(keyState,DIK_Q)) {
		if (!flags.q) {
			input.Q = 1;
			flags.q = 1;
		}
	} else {
		flags.q = 0;
	}

	if (DIKEYDOWN(keyState,DIK_W)) {
		if (!flags.w) {
			input.W = 1;
			flags.w = 1;
		}
	} else {
		flags.w = 0;
	}

	if (DIKEYDOWN(keyState,DIK_E)) {
		if (!flags.e) {
			input.E = 1;
			flags.e = 1;
		}
	} else {
		flags.e = 0;
	}


	if (DIKEYDOWN(keyState,DIK_SPACE)) input.SPACE = 1;

	if (mouseState.rgbButtons[0]) input.lButton = 1;
	input.x = mouseState.lX;
	input.y = mouseState.lY;



	if (DIKEYDOWN(keyState,DIK_1)) input.k1 = 1;
	if (DIKEYDOWN(keyState,DIK_2)) input.k2 = 1;
	if (DIKEYDOWN(keyState,DIK_3)) input.k3 = 1;
	if (DIKEYDOWN(keyState,DIK_4)) input.k4 = 1;
	if (DIKEYDOWN(keyState,DIK_5)) input.k5 = 1;
	if (DIKEYDOWN(keyState,DIK_6)) input.k6 = 1;
	if (DIKEYDOWN(keyState,DIK_7)) input.k7 = 1;
	if (DIKEYDOWN(keyState,DIK_8)) input.k8 = 1;
	if (DIKEYDOWN(keyState,DIK_9)) input.k9 = 1;
	if (DIKEYDOWN(keyState,DIK_0)) input.k0 = 1;

	return 0;
}

/*     */
int InitInput(void)
{
	/*   DirectInput */
	if (FAILED(DirectInput8Create(ghInst,DIRECTINPUT_VERSION,
		&IID_IDirectInput8,&lpdi,NULL)))
		return 1;

	/*    */
	if (FAILED(IDirectInput8_CreateDevice(lpdi,&GUID_SysKeyboard,
		&lpdiKey,NULL)))
		return 1;
	/*      */
	if (FAILED(IDirectInputDevice8_SetCooperativeLevel(lpdiKey,
		ghWndMain,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
		return 1;
	/*    */
	if (FAILED(IDirectInputDevice8_SetDataFormat(lpdiKey,&c_dfDIKeyboard)))
		return 1;
	/*   */
	if (FAILED(IDirectInputDevice8_Acquire(lpdiKey)))
		return 1;

	/*    */
	if (FAILED(IDirectInput8_CreateDevice(lpdi,&GUID_SysMouse,
		&lpdiMouse,NULL)))
		return 1;
	/*      */
	if (FAILED(IDirectInputDevice8_SetCooperativeLevel(lpdiMouse,
		ghWndMain,DISCL_BACKGROUND|DISCL_NONEXCLUSIVE)))
		return 1;
	/*    */
	if (FAILED(IDirectInputDevice8_SetDataFormat(lpdiMouse,&c_dfDIMouse)))
		return 1;
	/*   */
	if (FAILED(IDirectInputDevice8_Acquire(lpdiMouse)))
		return 1;

	return 0;
}

/*     */
void ReleaseInput(void)
{
	/*   */
	if (lpdiKey) {
		IDirectInputDevice8_Unacquire(lpdiKey);
		IDirectInputDevice8_Release(lpdiKey);
	}
	/*   */
	if (lpdiMouse) {
		IDirectInputDevice8_Unacquire(lpdiMouse);
		IDirectInputDevice8_Release(lpdiMouse);
	}

	/*   DirectInput */
	if (lpdi)
		IDirectInput8_Release(lpdi);
}
