/*
 * logic.c
 *
 * :       
 *       (256,192) 
 *    (0,0).  ,   'draw.h'
 *     , 
 *     ,    
 *    . ( 'global.h')
 */
#include "global.h"
#include <stdlib.h>
#include <math.h>
#include "logic.h"
#include "draw.h"
#include "input.h"
#include "sound.h"

/* pi */
#define PI                    3.14159265358979

/*      ( ) */
#define RACKET_X              (LOGIC_X+108)
#define RACKET_Y              (LOGIC_Y+165)

/*       */
#define START_SPEED_BALL            2.8
//#define START_SPEED               6.5
//#define START_SPEED               3.5

/*     */
#define SPEED_SLIVER                1.65

/*   */
#define SWERVE_R                   11      /*    */
#define BALL_R                      3      /*   */
/*    */
#define SUM_R           ((SWERVE_R + BALL_R) * (SWERVE_R + BALL_R))

/*      'y' */
#define ACCELERATION_Y                 0.029
/*     */
#define SPEED_Y                          1.9

/*    */
typedef struct {
	int on;                 /*   */
	int count;              /*     */
	int x,y;                /*    */
} SWERVE, *LPSWERVE;

/*       */
typedef struct {
	int x,y;                    /*     */
	int offset_x, offset_y;     /*     */
} BOUND, *LPBOUND;

/*   */
typedef struct {
	int on;                /*   */
	double v;              /*   */
	double vx,vy;          /*     'x'  'y' */
	double x,y;            /*  */
	int angle;             /*    ( ) */
	int ix,iy;             /*   */
	/*      */
	struct {
		int lock;          /*    */
		int angle;         /*      */
		int num;           /*      */
	} swerve;
} BALL, *LPBALL;

/*   */
typedef struct {
	int on;                          /*   */
	int type;                        /*   */
	int x,y;                         /*   */
} RACKET;

/*       */
typedef struct {
	int number;                     /*   */
	int count;                      /*    */
} FRAME;

/*    */
typedef struct {
	int on;                         /* 'alien'  */
	double v;                       /*  'alien' */
	double vx,vy;                   /*    'x'   'y' */
	double x,y;                     /*  */
	int angle;                      /*    ( ) */
	int ix,iy;                      /*   */
	/*   */
	struct {
		const FRAME *frames;        /*      */
		int point;                  /*    */
		int count;                  /*    */
	} animation;
	/*     */
	struct {
		int state;                  /*    */
		int count;                  /*     */
	} automat;
} ALIEN;

/*     */
typedef struct {
	int on;                         /*   */
	int point;                      /*    */
	int count;                      /*    */
} BANG;

/*   */
typedef struct {
	int on;                         /*   */
	double vy;                      /*   (   'y') */
	double y;                       /*  'y' */
	int ix,iy;                      /*   */
	int count;                      /*      */
} BOMB;

/*   */
typedef struct {
	int on;                         /*   */
	double vx,vy;                   /*     */
	double x,y;                     /*  */
	int ix,iy;                      /*   */
} BONUS;

/*    */
typedef struct {
	int state;                      /*    */
	int count;                      /*   */
	int (*func)(void);              /*      */
} ANIMA;

/*    */
typedef struct {
	double vx,vy;                   /*    'x'  'y' */
	double x,y;                     /*  */
} SLIVER, *LPSLIVER;

/*     */
static const FRAME lpAlienBird[] = {
	{0,3},{1,3},{2,3},{3,3},
	{4,3},{3,3},{2,3},{1,3},
	{-1,0}   /*   */
};

/*     */
static const FRAME lpAlienUfo[] = {
	{0,3},{1,3},{2,3},{3,3},{4,3},
	{5,3},{4,3},{3,3},{2,3},{1,3},
	{-1,0}   /*   */
};

/*     */
static const FRAME lpBang[] = {
	{0,2},{1,2},{2,2},{3,2},{4,2},
	{3,2},{2,2},{1,2},{0,2},
	{-1,0}   /*   */
};

/*
 *    
 * 0 -   
 * 1 -   
 * 2 -   
 * 3 -   
 * 4 -   
 */
static const FRAME lpFlySliverAnima[] = {
	{0,1},{0,35},{1,14},{2,7},{3,3},{4,2},{5,55},
	{-1,0}   /*   */
};

/*
 *   
 * 0 -  
 * 1 -      
 * 2 -  ,       
 */
static const FRAME lpStartRoundAnima[] = {
	{0,5},{1,60},{2,14},{-1,0}
};

/*
 *   
 * 0 -   3
 * 1 -   2
 * 2 -   1
 */
static const FRAME lpPauseEndAnima[] = {
	{0,35},{1,35},{2,35},{-1,0}
};

/*
 * 1 -  
 * 2 -  
 * 3 -  
 * 4 -  
 * 5 -  
 * 6 -  
 *
 *    :
 * 0x40 -     
 * 0x80 -   
 */
/*  1 (15,12) */
static const UCHAR round0[] = {
	0,0,0,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0,0,0,    /* 1 */
	0,0,2,5,5,5,5,5,5,5,5,5,2,0,0,                               /* 2 */
	0,2,2,2,5,5,5,5,5,5,5,2,2,2,0,                               /* 3 */
	2,2,0x43,2,2,4,4,4,4,4,2,2,0x43,2,2,                         /* 4 */
	5,2,2,2,1,1,1,1,1,1,1,2,2,2,5,                               /* 5 */
	5,5,2,1,1,1,1,1,1,1,1,1,2,5,5,                               /* 6 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                               /* 7 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                               /* 8 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                               /* 9 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                               /* 10 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                               /* 11 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0                                /* 12 */
};

/*  2 (15,12) */
static const UCHAR round1[] = {
	0,0,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0,0,   /* 1 */
	0,0x46,0x46,1,2,2,0,0,0,2,2,1,0x46,0x46,0,                        /* 2 */
	0x46,0x46,1,1,2,2,0,0,0,2,2,1,1,0x46,0x46,                        /* 3 */
	0x46,5,1,1,2,2,0,0,0,2,2,1,1,5,0x46,                              /* 4 */
	5,5,1,1,2,2,0,0,0,2,2,1,1,5,5,                                    /* 5 */
	5,5,1,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,0x46,1,5,5,         /* 6 */
	5,5,0x46,0x46,0,0,0,0,0,0,0,0x46,0x46,5,5,                        /* 7 */
	5,0x46,0x46,0,0,0,0,0,0,0,0,0,0x46,0x46,5,                        /* 8 */
	0x46,0x46,0,0,0,0,0,0,0,0,0,0,0,0x46,0x46,                        /* 9 */
	0x46,0,0,0,0,0,0,0,0,0,0,0,0,0,0x46,                              /* 10 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                                    /* 11 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0                                     /* 12 */
};

/*  3 (15,12) */
static const UCHAR round2[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,                                       /* 1 */
	0x46,0x46,0x46,0x46,0x46,0x83,0,0,0,0x83,0x46,0x46,0x46,0x46,0x46,   /* 2 */
	0,0,0,0,0,0x83,0,0,0,0x83,0,0,0,0,0,                                 /* 3 */
	0,0,0,0,0,0x83,0,0,0,0x83,0,0,0,0,0,                                 /* 4 */
	0,0x83,0x83,0x83,0x83,0x83,0,2,0,0x83,0x83,0x83,0x83,0x83,0,         /* 5 */
	0,0x83,1,0,0,0,2,2,2,0,0,0,1,0x83,0,                                 /* 6 */
	0,0x83,1,0,0,0,2,2,2,0,0,0,1,0x83,0,                                 /* 7 */
	0,0x83,0x83,0x83,0x83,0x83,0,2,0,0x83,0x83,0x83,0x83,0x83,0,         /* 8 */
	0,0,0,0,5,0x83,0,0,0,0x83,5,0,0,0,0,                                 /* 9 */
	0,0,0,0,5,0x83,0,0,0,0x83,5,0,0,0,0,                                 /* 10 */
	0x83,0x83,0x83,0x83,0x83,0x83,0,0,0,0x83,0x83,0x83,0x83,0x83,0x83,   /* 11 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0                                        /* 12 */
};

/*  4 (15,12) */
static const UCHAR round3[] = {
	0,0,4,4,2,2,0,0,0,2,2,4,4,0,0,              /* 1 */
	0,4,4,2,2,0x46,3,0,3,0x46,2,2,4,4,0,        /* 2 */
	0,4,2,2,0x46,3,3,0,3,3,0x46,2,2,4,0,        /* 3 */
	0,2,2,0x46,3,3,1,0,1,3,3,0x46,2,2,0,        /* 4 */
	0,2,0x46,3,3,1,5,0,5,1,3,3,0x46,2,0,        /* 5 */
	0,0x46,3,0,0,5,5,0,5,5,0,0,3,0x46,0,        /* 6 */
	0,3,3,0,0,5,0x46,0,0x46,5,0,0,3,3,0,        /* 7 */
	0,3,1,5,5,0x46,2,0,2,0x46,5,5,1,3,0,        /* 8 */
	0,1,5,5,0x46,2,2,0,2,2,0x46,5,5,1,0,        /* 9 */
	0,5,5,0x46,2,2,4,0,4,2,2,0x46,5,5,0,        /* 10 */
	0,5,0x46,2,2,4,4,0,4,4,2,2,0x46,5,0,        /* 11 */
	0,0,2,2,4,4,0,0,0,4,4,2,2,0,0               /* 12 */
};

/*  5 (15,12) */
static const UCHAR round4[] = {
	0x82,0x82,0x82,0x82,0,0,0x82,0x82,0x82,0,0,0x82,0x82,0x82,0x82,              /* 1 */
	0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,  /* 2 */
	0,0,0,0,0x82,0,0,0,0,0,0x82,0,0,0,0x82,                                      /* 3 */
	0,0,0,0,0x82,0,0,0,0,0,0x82,0,0,0,0x82,                                      /* 4 */
	0,0x82,3,0,0x82,0,5,0x82,4,0,0x82,0,0x82,0,0x82,                             /* 5 */
	0,0x82,0,0,0x82,0,0,0x82,0,0,0x82,1,0x82,1,0x82,                             /* 6 */
	0,0x82,0,0,0x82,0,0,0x82,0,0,0x82,0,0x82,0,0x82,                             /* 7 */
	0,0x82,0,3,0x82,5,0,0x82,0,4,0x82,0,0x82,0,0x82,                             /* 8 */
	0,0x82,0,0,0,0,0,0x82,0,0,0,0,0x82,0,0x82,                                   /* 9 */
	0,0x82,0,0,0,0,0,0x82,0,0,0,0,0x82,1,0x82,                                   /* 10 */
	0,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,     /* 11 */
	0,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82,0x82      /* 12 */
};

/*  6 (15,12) */
static const UCHAR round5[] = {
	0,0,0x43,0x43,0,0,0,0,0,0,0,0x43,0x43,0,0,                                /* 1 */
	0,0x43,0x43,0x43,0x43,0x43,0,0,0,0x43,0x43,0x43,0x43,0x43,0,              /* 2 */
	0,0x43,0x46,0x46,0x43,0x43,0x43,0,0x43,0x43,0x43,0x46,0x46,0x43,0,        /* 3 */
	0,0x43,0x46,1,0x46,0x43,0x43,0,0x43,0x43,0x46,1,0x46,0x43,0,              /* 4 */
	0,0x43,0x43,0x46,0x46,0x43,0x43,0x43,0x43,0x43,0x46,0x46,0x43,0x43,0,     /* 5 */
	0,0,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0,0,           /* 6 */
	0,0,0,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0x43,0,0,0,                 /* 7 */
	0,0,0,0,0x43,0x43,1,0x43,1,0x43,0x43,0,0,0,0,                             /* 8 */
	0,0,0x43,0x43,0x43,0x43,1,0,1,0x43,0x43,0x43,0x43,0,0,                    /* 9 */
	0,0x43,0x43,0x43,0x43,0,1,0,1,0,0x43,0x43,0x43,0x43,0,                    /* 10 */
	0x43,0x43,0x43,0,0,0,0,0,0,0,0,0,0x43,0x43,0x43,                          /* 11 */
	0x43,0x43,0,0,0,0,0,0,0,0,0,0,0,0x43,0x43                                 /* 12 */
};

/*   () (15,12) */
static const UCHAR round10[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 1 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 2 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 3 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 4 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 5 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 6 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 7 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 8 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 9 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 10 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,        /* 11 */
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0         /* 12 */
};

/*    */
typedef struct {
	/*
	 *     
	 *   .
	 *
	 * 0  -   
	 * -1 -  
	 */
	int AlienBlock;
	/*     */
	int swer0,swer0X,swer0Y;
	/*     */
	int swer1,swer1X,swer1Y;
	/*     */
	int swer2,swer2X,swer2Y;
	/*     */
	int swer3,swer3X,swer3Y;
	/*     */
	const UCHAR *round;
} LEVEL, *LPLEVEL;

/*   */
#define LEVEL_COUNT                 10

/*    */
static const LEVEL levels[LEVEL_COUNT] = {
	{0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round0},
	{-1, 1,116,44, 0,0,0, 0,0,0, 0,0,0, round1},
	{-1, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round2},
	{-1, 1,116,124, 0,0,0, 0,0,0, 0,0,0, round3},
	{0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round4},
	{20, 1,116,16, 1,72,115, 1,160,115, 0,0,0, round5},
	{0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round10},
	{-1, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round10},
	{-1, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round10},
	{0, 0,0,0, 0,0,0, 0,0,0, 0,0,0, round10}
};

/*    */
static int nLevel;
/*     (15,16) */
static UCHAR block[16][15];
/*       */
static int EventBlock[16][15];
/*   */
static SWERVE swerve[SWERVES_COUNT];
/*  */
static BALL ball[BALL_COUNT];
/*  */
static RACKET racket;
/*   */
static ALIEN alien;
/*  */
static BANG bang;
/*  */
static BOMB bomb;
/*  */
static BONUS bonus;
/*   */
static ANIMA anima;
/*   */
static SLIVER sliver[SLIVERS_COUNT];


/*   */
static int hand;            /*  */
static int kill;            /*    */
/*   */
static struct _PAUSE {
	int deny;               /*      */
	int pause;              /*   */
} pause;

static int slip;




/*     */
static int ftol(double a)
{
	/*    */
	return (int)a;
}

/*       */
static int random(int range)
{
	return (range * rand() / (RAND_MAX + 1));
}

/*      */
static void CopyBlock(void)
{
	int i,j;

	UCHAR *to,*from;
	UCHAR symbol;
	int *event;

	/*    */
	to = &scene.block[0][0];
	from = &block[3][0];
	event = &EventBlock[3][0];
	for (i = 0; i < 12; i++) {
		/*    */
		for (j = 0; j < 15; j++) {
			int temp = event[j];
			symbol = from[j] & 0x0f;
			/*      */
			if (temp) {
				/*   */
				if (temp <= 2) {
					to[j] = 0x10 | symbol;
					event[j]++;
				} else if (temp <= 4) {
					to[j] = 0x20 | symbol;
					event[j]++;
				} else if (temp <= 6) {
					to[j] = 0x30 | symbol;
					event[j]++;
				} else if (temp <= 8) {
					to[j] = 0x40 | symbol;
					event[j]++;
				} else if (temp <= 10) {
					to[j] = 0x50 | symbol;
					event[j]++;
				} else if (temp <= 14) {
					to[j] = 0x60 | symbol;
					event[j]++;
				} else {
					/*   */
					to[j] = symbol;
					event[j] = 0;
				}
			} else {
				/*  ,   */
				to[j] = symbol;
			}
		}
		/*   */
		to += 15;
		from += 15;
		event += 15;
	}
}

/*    */
static int GetWidthBonus(void)
{
	int width;

	switch (scene.pBonus.type) {
		case BONUS_POINTS:
			width = 23;
			break;
		case BONUS_SMART_BOMB:
			width = 24;
			break;
		case BONUS_EXTRA_LIFE:
			width = 19;
			break;
		case BONUS_SLOW_BALL:
			width = 22;
			break;
		case BONUS_SMASH_BALL:
			width = 23;
			break;
		case BONUS_TRIPLE_BALL:
			width = 17;
			break;
		case BONUS_HAND:
			width = 22;
			break;
		case BONUS_GUN:
			width = 20;
			break;
		case BONUS_EXTENDED_RACKET:
			width = 22;
			break;
		case BONUS_ROCKET_PACK:
			width = 21;
			break;
		case BONUS_400_POINTS:
			width = 22;
			break;
		default:
			width = 0;
	}

	return width;
}

/*    */
static int GetHeightBonus(void)
{
	int height;

	switch (scene.pBonus.type) {
		case BONUS_POINTS:
			height = 14;
			break;
		case BONUS_SMART_BOMB:
			height = 15;
			break;
		case BONUS_EXTRA_LIFE:
			height = 13;
			break;
		case BONUS_SLOW_BALL:
			height = 15;
			break;
		case BONUS_SMASH_BALL:
			height = 15;
			break;
		case BONUS_TRIPLE_BALL:
			height = 15;
			break;
		case BONUS_HAND:
			height = 12;
			break;
		case BONUS_GUN:
			height = 9;
			break;
		case BONUS_EXTENDED_RACKET:
			height = 11;
			break;
		case BONUS_ROCKET_PACK:
			height = 12;
			break;
		case BONUS_400_POINTS:
			height = 10;
			break;
		default:
			height = 0;
	}

	return height;
}

/*      */
static int GetRandomBonus(void)
{
	return BONUS_TRIPLE_BALL;
	return random(10);
}

/*     */
static void CreateBonus(int x, int y)
{
	/*      */
	if (bonus.on)
		return;

	/*     10% */
	if (random(1))
		return;

	/*   */
	bonus.on = 1;
	bonus.ix = x * 16 + LOGIC_X - ((GetWidthBonus() - 16) / 2);
	bonus.iy = y * 8 + LOGIC_Y;
	bonus.x = bonus.ix;
	bonus.y = bonus.iy;
	bonus.vy = 0.0;
	/*   */
	scene.pBonus.show = 1;
	scene.pBonus.X = bonus.ix;
	scene.pBonus.Y = bonus.iy;
	scene.pBonus.type = GetRandomBonus();
}

/*     */
static void MarkBlock(int x, int y)
{
	UCHAR *point;
	int attr;

	/*   */
	point = &block[y][x];
	if (*point) {
		attr = *point & 0xf0;
		if (attr == 0x80) {
			/*    */
			EventBlock[y][x] = 1;
			/*   */
			play.stone = 1;
		} else if (attr == 0x40) {
			/*      */
			EventBlock[y][x] = 1;
			*point &= 0x0f;
			/*   */
			play.stone = 1;
		} else {
			/*   */
			*point = 0;
			/*   */
			EventBlock[y][x] = 0;
			/*     */
			CreateBonus(x,y);
			/*   */
			play.brik = 1;
		}
	}
}

/*       */
static int GetCountBlock(void)
{
	int i,j,count;
	UCHAR *p;

	/*   */
	count = 0;
	p = &block[3][0];
	/*    */
	for (i = 0; i < 12; i++) {
		for (j = 0; j < 15; j++)
			if (p[j]) count++;
		p += 15;
	}

	return count;
}

/*     'x'  'y'   */
static void SetSpeedBall(LPBALL lpBall)
{
	double rad;

	/*     */
	rad = lpBall->angle * PI / 180.0;
	/*    'x'  'y' */
	lpBall->vx = lpBall->v * cos(rad);
	lpBall->vy = lpBall->v * sin(rad);
}

/*    ,     */
static int GetBlock(LPBOUND bound, int px, int py)
{
	int x,y;

	/*      */
	x = px - LOGIC_X;
	y = py - LOGIC_Y;
	if ((x < 0) || (y < 0))
		return 0;
	x /= 16;
	y /= 8;
	if ((x >= 15) || (y >= 16))
		return 0;

	/* ,    */
	if (!block[y][x])
		return 0;

	/*      */
	bound->x = x;
	bound->y = y;
	bound->offset_x = px - (x * 16 + LOGIC_X);
	bound->offset_y = py - (y * 8 + LOGIC_Y);

	return 1;
}

/*      */
static void SetRacket(void)
{
	/*       */
	racket.on = 1;
	racket.type = TYPE_RACKET_NORMAL;
	racket.x = RACKET_X;
	racket.y = RACKET_Y;
	/*   */
	scene.pRacket.show = 1;
	scene.pRacket.type = racket.type;
	scene.pRacket.X = racket.x;
	scene.pRacket.Y = racket.y;
}

/*      */
static void SetBall(void)
{
	/*       */
	ZeroMemory(scene.pBall,sizeof(scene.pBall));
	ZeroMemory(ball,sizeof(ball));

	/*      */
	scene.pBall[0].picture.show = 1;
	scene.pBall[0].picture.X = racket.x + 12;
	scene.pBall[0].picture.Y = racket.y - 6;
	/*     */
	scene.pBall[0].shadow.show = 1;
	scene.pBall[0].shadow.X = scene.pBall[0].picture.X + 7;
	scene.pBall[0].shadow.Y = scene.pBall[0].picture.Y + 7;

	/*     */
	ball[0].on = 1;
	ball[0].angle = 293;
	ball[0].v = START_SPEED_BALL;
	SetSpeedBall(&ball[0]);
	ball[0].x = scene.pBall[0].picture.X;
	ball[0].y = scene.pBall[0].picture.Y;
}

/*       */
static void SetSwerve(void)
{
	int i;
	const LEVEL *lpLevel;

	/*       */
	ZeroMemory(scene.pSwerves,sizeof(scene.pSwerves));
	ZeroMemory(swerve,sizeof(swerve));

	/*    */
	lpLevel = &levels[nLevel%LEVEL_COUNT];

	/*    */
	for (i = 0; i < SWERVES_COUNT; i++) {
		LPSWERVE lpSwerve;
		LPPICTURE lpPicture;
		int flags,X,Y;

		/*   ,   */
		switch (i) {
			case 0:
				flags = lpLevel->swer0;
				X = lpLevel->swer0X;
				Y = lpLevel->swer0Y;
				break;

			case 1:
				flags = lpLevel->swer1;
				X = lpLevel->swer1X;
				Y = lpLevel->swer1Y;
				break;

			case 2:
				flags = lpLevel->swer2;
				X = lpLevel->swer2X;
				Y = lpLevel->swer2Y;
				break;

			case 3:
				flags = lpLevel->swer3;
				X = lpLevel->swer3X;
				Y = lpLevel->swer3Y;
				break;

			default:
				continue;
		}

		/*   */
		lpSwerve = &swerve[i];
		lpPicture = &scene.pSwerves[i];

		/*     */
		if (flags) {
			/*   */
			lpPicture->show = 1;
			/*   */
			if (random(2)) {
				lpPicture->type = PICTURE_SWERVE_ON;
				lpSwerve->on = 1;
			} else {
				lpPicture->type = PICTURE_SWERVE_OFF;
				lpSwerve->on = 0;
			}
			/*   */
			lpPicture->X = X;
			lpPicture->Y = Y;
			/*      */
			lpSwerve->x = X + 12;
			lpSwerve->y = Y + 12;
			lpSwerve->count = 30 + random(25);
		}
	}
}

/*   ,     */
static void CreateBang(void)
{
	/*    */
	alien.on = 0;
	scene.pAlien.show = 0;
	/*    */
	bang.on = 1;
	bang.count = 0;
	bang.point = 0;
	scene.pBang.show = 1;
	scene.pBang.type = lpBang[bang.point].number;
	scene.pBang.X = alien.ix + 4;
	scene.pBang.Y = alien.iy - 3;
}

/*    */
static int StartRoundAnima(void)
{
	int state;

	/*    */
	if (++anima.count < lpStartRoundAnima[anima.state].count)
		return 1;

	/*   */
	anima.count = 0;

	/*    */
	state = lpStartRoundAnima[++anima.state].number;
	if (state == 1) {
		/*   */
		scene.on = 1;
		/*    */
		SetSwerve();
		/*   */
		scene.message.show = 1;
		scene.message.flags = MESSAGE_ROUND;
		scene.message.player = 1;
		scene.message.round = nLevel + 1;
	} else if (state == 2) {
		UCHAR *b;
		int *e,i,j;
		
		/*   ,    */
		b = &block[3][0];
		e = &EventBlock[3][0];
		for (i = 0; i < 12; i++) {
			for (j = 0; j < 15; j++) {
				if (b[j] & (0x80 | 0x40))
					e[j] = 1;
			}
			b += 15;
			e += 15;
		}
	} else {
		/*   */
		anima.func = NULL;
		/*   */
		scene.message.show = 0;
		/*   */
		pause.deny = 0;
		/*   */
		SetRacket();
		/*   */
		SetBall();

		hand = 1;
	}

	return 1;
}

/*     */
static void SetStartRoundAnima(void)
{
	int i;

	/*    */
	anima.func = StartRoundAnima;
	anima.count = 0;
	anima.state = 0;
	/*   */
	pause.deny = 1;
	/*    */
	scene.on = 0;
	/*   */
	racket.on = 0;
	scene.pRacket.show = 0;
	/*   */
	for (i = 0; i < BALL_COUNT; i++) {
		ball[i].on = 0;
		scene.pBall[i].picture.show = 0;
		scene.pBall[i].shadow.show = 0;
	}
	/*    */
	alien.on = 0;
	scene.pAlien.show = 0;
	/*   */
	bomb.on = 0;
	scene.pBomb.picture.show = 0;
	scene.pBomb.shadow.show = 0;
	/*   */
	bang.on = 0;
	scene.pBang.show = 0;
}

/*    */
static int PauseEndAnima(void)
{
	int state;

	/*    */
	if (++anima.count < lpPauseEndAnima[anima.state].count)
		return 2;

	/*   */
	anima.count = 0;

	/*    */
	state = lpPauseEndAnima[++anima.state].number;
	/*     */
	if (state == 1) {
		scene.message.pause = 2;
	} else if (state == 2) {
		scene.message.pause = 1;
	} else {
		/*   */
		anima.func = NULL;
		/*   */
		scene.message.show = 0;
		/*   */
		pause.deny = 0;
	}

	return 2;
}

/*     */
static void SetPauseEndAnima(void)
{
	/*    */
	anima.func = PauseEndAnima;
	anima.count = 0;
	anima.state = 0;
	/*   */
	pause.deny = 1;
	/*    */
	scene.message.show = 1;
	scene.message.flags = MESSAGE_PAUSE;
	scene.message.pause = 3;
}



//////////////////////////////////////
/*    */
static void CheckControlKey(void)
{
	static int flags;
	int i;

	if (input.SPACE || input.lButton) {
		if (!flags) {
			hand = 0;
			flags = 1;

			for (i = 0; i < BALL_COUNT; i++) {
				if (scene.pBall[i].picture.show) {
					ball[i].on = 1;
					ball[i].angle = 293;
					SetSpeedBall(&ball[i]);
				}
			}
		}
	} else {
		flags = 0;
	}

	if (input.Q) {
		/*    */
		CopyMemory(&block[3][0],levels[nLevel].round,sizeof(UCHAR)*(15*12));
	}

	if (input.W) {
		slip = ~slip;
	}

	if (input.E) {
		if (!pause.deny) {
			if (!pause.pause) {
				scene.message.show = 1;
				scene.message.flags = MESSAGE_PAUSE;
				scene.message.pause = 0;
				pause.pause = 1;
			} else {
				/*   */
				pause.pause = 0;
				/*     */
				SetPauseEndAnima();
			}
		}
	}


	if (input.k1) {
		racket.type = TYPE_RACKET_NORMAL;
		scene.pRacket.type = TYPE_RACKET_NORMAL;
	} else if (input.k2) {
		racket.type = TYPE_RACKET_GUN;
		scene.pRacket.type = TYPE_RACKET_GUN;
	} else if (input.k3) {
		racket.type = TYPE_RACKET_EXTEND;
		scene.pRacket.type = TYPE_RACKET_EXTEND;
	} else if (input.k4) {
		scene.pBonus.X--;
	} else if (input.k5) {
		scene.pBonus.X++;
	} else if (input.k6) {
		scene.pBonus.Y--;
	} else if (input.k7) {
		scene.pBonus.Y++;
	} else if (input.k8) {
	} else if (input.k9) {
	} else if (input.k0) {
	}
}
//////////////////////////////////////



/*    */
static int GetWidthRacket(void)
{
	int width;

	/*        */
	switch (racket.type) {
		case TYPE_RACKET_N_G_1:
		case TYPE_RACKET_N_G_2:
			width = 26;
			break;
		case TYPE_RACKET_N_E_1:
			width = 30;
			break;
		case TYPE_RACKET_N_E_2:
			width = 32;
			break;
		case TYPE_RACKET_N_E_3:
			width = 34;
			break;
		case TYPE_RACKET_N_E_4:
			width = 36;
			break;
		case TYPE_RACKET_N_E_5:
			width = 38;
			break;
		case TYPE_RACKET_N_E_6:
			width = 40;
			break;
		case TYPE_RACKET_N_E_7:
			width = 42;
			break;
		case TYPE_RACKET_EXTEND:
			width = 44;
			break;
		case TYPE_RACKET_NORMAL:
		case TYPE_RACKET_GUN:
		case TYPE_RACKET_N_G_3:
		case TYPE_RACKET_N_G_4:
		default:
			width = 28;
			break;
	}

	return width;
}

/*    */
static int GetHeightRacket(void)
{
	/*      10 */
	return 10;
}

/*    */
static int GetWidthBall(void)
{
	/*     */
	return 7;
}

/*   */
static void MoveRacket(void)
{
	int width;

	/* ,     */
	if (!racket.on)
		return;

	/*    */
	width = GetWidthRacket();

	/*     */
	if (input.K) {
		/*      */
		racket.x -= 4;
		if (racket.x < LOGIC_X)
			racket.x = LOGIC_X;
	}
	if (input.L) {
		/*      */
		racket.x += 4;
		if (racket.x > (LOGIC_X + LOGIC_WIDTH - width))
			racket.x = LOGIC_X + LOGIC_WIDTH - width;
	}

	/*     */
	if (input.x) {
		/*   */
		racket.x += ftol(input.x*0.3);
		if (racket.x < LOGIC_X)
			racket.x = LOGIC_X;
		else if (racket.x > (LOGIC_X + LOGIC_WIDTH - width))
			racket.x = LOGIC_X + LOGIC_WIDTH - width;
	}


	/*////////////////////////////
	if (input.y) {
		racket.y += ftol(input.y*0.3);
		scene.pRacket.Y = racket.y;
	}
	////////////////////////////*/


	/*   */
	scene.pRacket.X = racket.x;
}

/*     */
static int MoveSliver(void)
{
	int i,offset;

	/*    */
	if (++anima.count >= lpFlySliverAnima[anima.state].count) {
		int state;

		/*    */
		anima.count = 0;
		/*      */
		state = lpFlySliverAnima[++anima.state].number;
		/*    */
		if (state == 5) {
			/*     */
			scene.pSlivers.show = 0;
			for (i = 0; i < SLIVERS_COUNT; i++) {
				LPFULL_PICTURE lpPicture = &scene.pSlivers.picture[i];
				lpPicture->picture.show = 0;
				lpPicture->shadow.show = 0;
			}
		} else if (state == -1) {
			/*   */
			pause.deny = 0;
			/*     */
			anima.func = NULL;
			/*     */
			SetStartRoundAnima();
			/*    */
			return 2;
		} else {
			/*     */
			for (i = 0; i < SLIVERS_COUNT; i++) {
				LPFULL_PICTURE lpPicture = &scene.pSlivers.picture[i];
				lpPicture->picture.type = state;
				lpPicture->shadow.type = state;
			}
		}
	}

	/*     */
	if (lpFlySliverAnima[anima.state].number == 5)
		return 1;

	/*     */
	switch (lpFlySliverAnima[anima.state].number) {
		case 0:
			offset = 6;
			break;
		case 1:
		case 2:
			offset = 5;
			break;
		case 3:
			offset = 4;
			break;
		default:
			offset = 3;
	}

	/*   */
	for (i = 0; i < SLIVERS_COUNT; i++) {
		LPFULL_PICTURE lpPicture = &scene.pSlivers.picture[i];
		LPSLIVER lpSliver = &sliver[i];
		int x,y;
		/*    */
		lpSliver->x += lpSliver->vx;
		lpSliver->y += lpSliver->vy;
		/*
		 *    .
		 *      , ..
		 *      .   
		 *   ,     
		 *     ( 'y').
		 */
		if (lpSliver->x < LOGIC_X) {
			lpSliver->vx = -lpSliver->vx;
			lpSliver->x = LOGIC_X;
		} else if (lpSliver->x > LOGIC_X + LOGIC_WIDTH - 7) {
			lpSliver->vx = -lpSliver->vx;
			lpSliver->x = LOGIC_X + LOGIC_WIDTH - 7;
		}
		/*    */
		x = ftol(lpSliver->x);
		y = ftol(lpSliver->y);
		lpPicture->picture.X = x;
		lpPicture->picture.Y = y;
		lpPicture->shadow.X = x + offset;
		lpPicture->shadow.Y = y + offset;
	}

	return 1;
}

/*      */
static int FlySliverAnima(void)
{
	/* ,     */
	if (!anima.state) {
		/*     */
		int i,type,racketWidth;
		double x,y,x0,y0,angle;

		/*     */
		anima.state++;
		/*    */
		type = lpFlySliverAnima[anima.state].number;
		/*
		 *     ,
		 *     .
		 *    a = 15, b = -5.
		 */
		/*    x,y */
		racketWidth = GetWidthRacket();
		if (racketWidth < 36)
			x = racket.x - ((36 - racketWidth) / 2);
		else if (racketWidth > 36)
			x = racket.x + ((racketWidth - 36) / 2);
		else
			x = racket.x;
		y = RACKET_Y;
		/*    x0,y0 */
		x0 = x + 15.0;
		y0 = y + 5.0;
		/*   */
		angle = 180.0;
		/*    */
		for (i = 0; i < SLIVERS_COUNT; i++) {
			LPFULL_PICTURE lpPicture;
			LPSLIVER lpSliver;
			double rad;
			int lx,ly;
			/*   x */
			x += 3.0;
			/*   'y'    */
			y = -5.0 * sqrt(fabs(1.0 - pow((x - x0) / 15.0,2.0))) + y0;
			/*    */
			lx = ftol(x);
			/*   x */
			if (lx < LOGIC_X)
				lx = LOGIC_X;
			else if (lx > LOGIC_X + LOGIC_WIDTH - 7)
				lx = LOGIC_X + LOGIC_WIDTH - 7;
			ly = ftol(y);
			/*     */
			lpSliver = &sliver[i];
			lpSliver->x = lx;
			lpSliver->y = ly;
			/*   (vx,vy)   */
			rad = angle * PI / 180.0;
			lpSliver->vx = SPEED_SLIVER * cos(rad);
			lpSliver->vy = SPEED_SLIVER * sin(rad);
			/*   */
			angle += 22.5;
			/*      */
			lpPicture = &scene.pSlivers.picture[i];
			lpPicture->picture.show = 1;
			lpPicture->picture.type = type;
			lpPicture->picture.X = lx;
			lpPicture->picture.Y = ly;
			/*        */
			lpPicture->shadow.show = 1;
			lpPicture->shadow.type = type;
			lpPicture->shadow.X = lx + 6;
			lpPicture->shadow.Y = ly + 6;
		}
		/*   */
		scene.pSlivers.show = 1;
	} else {
		/*   */
		return MoveSliver();
	}

	return 1;
}

/*   */
static void DestroyRacket(void)
{
	int i;

	/*      */
	anima.func = FlySliverAnima;
	anima.count = 0;
	anima.state = 0;
	/*   */
	pause.deny = 1;
	/*   */
	racket.on = 0;
	scene.pRacket.show = 0;
	/*   */
	for (i = 0; i < BALL_COUNT; i++) {
		ball[i].on = 0;
		scene.pBall[i].picture.show = 0;
		scene.pBall[i].shadow.show = 0;
	}
	/*    */
	alien.on = 0;
	scene.pAlien.show = 0;
	/*   */
	bomb.on = 0;
	scene.pBomb.picture.show = 0;
	scene.pBomb.shadow.show = 0;
	/*   */
	bang.on = 0;
	scene.pBang.show = 0;
	/*   */
	bonus.on = 0;
	scene.pBonus.show = 0;

	/*   */
	play.destroy = 1;
}

/*     */
static int LockSwerve(LPSWERVE lpSwerve, LPBALL lpBall)
{
	int flags = 0;

	/*        */
	switch (lpBall->angle) {
		/*     */
		case 293:
			/* ,      */
			if (lpBall->iy + 3 <= lpSwerve->y + 1) {
				/*    */
				flags = 1;
				/*
				 *     
				 *   .
				 */
				if (lpBall->ix + 3 <= lpSwerve->x - 9) {
					lpBall->swerve.angle = 315;
					lpBall->angle = 304;
				} else {
					lpBall->swerve.angle = 247;
					lpBall->angle = 270;
				}
			}
			break;

		case 247:
			if (lpBall->iy + 3 <= lpSwerve->y + 1) {
				flags = 1;
				if (lpBall->ix + 3 >= lpSwerve->x + 9) {
					lpBall->swerve.angle = 225;
					lpBall->angle = 236;
				} else {
					lpBall->swerve.angle = 293;
					lpBall->angle = 270;
				}
			}
			break;

		case 113:
			if (lpBall->iy + 3 >= lpSwerve->y - 1) {
				flags = 1;
				if (lpBall->ix + 3 >= lpSwerve->x + 9) {
					lpBall->swerve.angle = 135;
					lpBall->angle = 124;
				} else {
					lpBall->swerve.angle = 67;
					lpBall->angle = 90;
				}
			}
			break;

		case 67:
			if (lpBall->iy + 3 >= lpSwerve->y - 1) {
				flags = 1;
				if (lpBall->ix + 3 <= lpSwerve->x - 9) {
					lpBall->swerve.angle = 45;
					lpBall->angle = 56;
				} else {
					lpBall->swerve.angle = 113;
					lpBall->angle = 90;
				}
			}
			break;

		/*     */
		case 203:
			/* ,      */
			if (lpBall->ix + 3 <= lpSwerve->x + 1) {
				/*    */
				flags = 1;
				/*
				 *     
				 *   .
				 */
				if (lpBall->iy + 3 >= lpSwerve->y + 9) {
					lpBall->swerve.angle = 225;
					lpBall->angle = 214;
				} else if (lpBall->iy + 3 >= lpSwerve->y - 1) {
					lpBall->swerve.angle = 247;
					lpBall->angle = 225;
				} else {
					lpBall->swerve.angle = 157;
					lpBall->angle = 180;
				}
			}
			break;

		case 157:
			if (lpBall->ix + 3 <= lpSwerve->x + 1) {
				flags = 1;
				if (lpBall->iy + 3 <= lpSwerve->y - 9) {
					lpBall->swerve.angle = 135;
					lpBall->angle = 146;
				} else if (lpBall->iy + 3 <= lpSwerve->y + 1) {
					lpBall->swerve.angle = 113;
					lpBall->angle = 135;
				} else {
					lpBall->swerve.angle = 203;
					lpBall->angle = 180;
				}
			}
			break;

		case 337:
			if (lpBall->ix + 3 >= lpSwerve->x - 1) {
				flags = 1;
				if (lpBall->iy + 3 >= lpSwerve->y + 9) {
					lpBall->swerve.angle = 315;
					lpBall->angle = 326;
				} else if (lpBall->iy + 3 >= lpSwerve->y - 1) {
					lpBall->swerve.angle = 293;
					lpBall->angle = 315;
				} else {
					lpBall->swerve.angle = 23;
					lpBall->angle = 0;
				}
			}
			break;

		case 23:
			if (lpBall->ix + 3 >= lpSwerve->x - 1) {
				flags = 1;
				if (lpBall->iy + 3 <= lpSwerve->y - 9) {
					lpBall->swerve.angle = 45;
					lpBall->angle = 34;
				} else if (lpBall->iy + 3 <= lpSwerve->y + 1) {
					lpBall->swerve.angle = 67;
					lpBall->angle = 45;
				} else {
					lpBall->swerve.angle = 337;
					lpBall->angle = 0;
				}
			}
			break;

		/*     */
		case 315:
			/* ,        */
			if ((lpBall->ix + 3 >= lpSwerve->x - 7) &&
				(lpBall->iy + 3 <= lpSwerve->y + 7)) {
				/*    */
				flags = 1;
				/*
				 *     
				 *   .
				 */
				if (lpBall->iy + 3 <= lpSwerve->y - 1) {
					lpBall->swerve.angle = 337;
					lpBall->angle = 326;
				} else {
					lpBall->swerve.angle = 293;
					lpBall->angle = 304;
				}
			}
			break;

		case 225:
			if ((lpBall->ix + 3 <= lpSwerve->x + 7) &&
				(lpBall->iy + 3 <= lpSwerve->y + 7)) {
				flags = 1;
				if (lpBall->iy + 3 <= lpSwerve->y - 1) {
					lpBall->swerve.angle = 203;
					lpBall->angle = 214;
				} else {
					lpBall->swerve.angle = 247;
					lpBall->angle = 236;
				}
			}
			break;

		case 45:
			if ((lpBall->ix + 3 >= lpSwerve->x - 7) &&
				(lpBall->iy + 3 >= lpSwerve->y - 7)) {
				flags = 1;
				if (lpBall->iy + 3 >= lpSwerve->y + 1) {
					lpBall->swerve.angle = 23;
					lpBall->angle = 34;
				} else {
					lpBall->swerve.angle = 67;
					lpBall->angle = 56;
				}
			}
			break;

		case 135:
			if ((lpBall->ix + 3 <= lpSwerve->x + 7) &&
				(lpBall->iy + 3 >= lpSwerve->y - 7)) {
				flags = 1;
				if (lpBall->iy + 3 >= lpSwerve->y + 1) {
					lpBall->swerve.angle = 157;
					lpBall->angle = 146;
				} else {
					lpBall->swerve.angle = 113;
					lpBall->angle = 124;
				}
			}
			break;
	}

	/*     */
	if (flags) {
		/*       */
		lpBall->swerve.lock = 1;
		/*   */
		SetSpeedBall(lpBall);
	}

	return flags;
}

/*     */
static void UnlockSwerve(LPBALL lpBall)
{
	/*   */
	if (lpBall->swerve.lock) {
		lpBall->swerve.lock = 0;
		/*       */
		lpBall->angle = lpBall->swerve.angle;
		/*   */
		SetSpeedBall(lpBall);
	}
}

/*      */
static int CheckCollideSwerve(LPBALL lpBall)
{
	int flags = 0;
	int i;

	/*      */
	for (i = 0; i < SWERVES_COUNT; i++) {
		LPSWERVE lpSwerve = &swerve[i];
		/*     */
		if (lpSwerve->on) {
			int dx = (lpBall->ix + 3) - lpSwerve->x;
			int dy = (lpBall->iy + 3) - lpSwerve->y;
			/* ,       */
			if ((dx * dx + dy * dy) <= SUM_R) {
				/*    ,    */
				if (!lpBall->swerve.lock) {
					/*        */
					if (LockSwerve(lpSwerve,lpBall)) {
						lpBall->swerve.num = i;
						flags = 1;
					}
				}
			} else {
				/*      ,    */
				if (lpBall->swerve.lock && (lpBall->swerve.num == i)) {
					UnlockSwerve(lpBall);
					flags = 1;
				}
			}
		} else {
			/*    ,   */
			if (lpBall->swerve.lock && (lpBall->swerve.num == i)) {
				UnlockSwerve(lpBall);
				/*    */
				flags = 1;
			}
		}
	}

	return flags;
}

/*     */
static int CheckCollideBorder(LPBALL lpBall)
{
	int flags = 0;

	/*     ,  'y' */
	if ((lpBall->ix <= LOGIC_X) &&
		(lpBall->angle > 90) && (lpBall->angle < 270)) {
		flags = 1;
		/*     ,    */
		UnlockSwerve(lpBall);
		/*      'y' */
		lpBall->angle = 180 - lpBall->angle;
		if (lpBall->angle < 0) lpBall->angle += 360;
		/*    */
		SetSpeedBall(lpBall);
		/*   'x' */
		lpBall->ix = LOGIC_X;
		/*   */
		play.wall = 1;
	} else if ((lpBall->ix >= (LOGIC_X + LOGIC_WIDTH - 7)) &&
		((lpBall->angle > 270) || (lpBall->angle < 90))) {
		flags = 1;
		/*     ,    */
		UnlockSwerve(lpBall);
		/*      'y' */
		lpBall->angle = 180 - lpBall->angle;
		if (lpBall->angle < 0) lpBall->angle += 360;
		/*    */
		SetSpeedBall(lpBall);
		/*   'x' */
		lpBall->ix = LOGIC_X + LOGIC_WIDTH - 7;
		/*   */
		play.wall = 1;
	}

	/*     ,  'x' */
	if ((lpBall->iy <= LOGIC_Y) && (lpBall->angle > 180)) {
		flags = 1;
		/*     ,    */
		UnlockSwerve(lpBall);
		/*      'x' */
		lpBall->angle = 360 - lpBall->angle;
		/*    */
		SetSpeedBall(lpBall);
		/*   'y' */
		lpBall->iy = LOGIC_Y;
		/*   */
		play.wall = 1;
	} else if (lpBall->iy > (LOGIC_Y + LOGIC_HEIGHT)) {
		/*      */
		int i,count;

		/*    */
		lpBall->on = 0;
		/*     */
		for (i = count = 0; i < BALL_COUNT; i++)
			if (!ball[i].on)
				count++;
		/*   ,   */
		if (count == BALL_COUNT)
			DestroyRacket();
	}

	return flags;
}

/*     */
static int CheckCollideRacket(LPBALL lpBall)
{
	int b1,b2,r1,r2,r3,r4,r5,r6;
	int wRacket,wBall,temp;

	/* ,     'y'  */
	if (lpBall->angle > 180)
		return 0;
	/*    */
	wBall = GetWidthBall();
	/* ,       */
	if ((lpBall->iy < (RACKET_Y - wBall)) || (lpBall->iy > (RACKET_Y-2)))
		return 0;
	/*    */
	wRacket = GetWidthRacket();
	/* ,      */
	if (((lpBall->ix + wBall) <= racket.x) ||
		(lpBall->ix >= (racket.x + wRacket)))
		return 0;

	/*      */
	temp = wRacket / 2;
	r3 = temp - 1;
	r4 = wRacket - (r3 + 1);
	r2 = temp / 2;
	r5 = wRacket - (r2 + 1);
	r1 = r2 / 2 + 1;
	r6 = wRacket - (r1 + 1);
	/*    */
	r1 += racket.x;
	r2 += racket.x;
	r3 += racket.x;
	r4 += racket.x;
	r5 += racket.x;
	r6 += racket.x;

	/*    ,    */
	b1 = lpBall->ix + 2;
	b2 = lpBall->ix + 4;

	/*
	 *     .
	 *      'x'.
	 */
	if (lpBall->angle < 90) {
		/*    'x' . */
		/*
		 *      
		 *    .
		 */
		if (b2 <= r1) {
			if (lpBall->angle == 67) lpBall->angle = 225;
			else lpBall->angle = 203;
		} else if ((b2 <= r2) && (lpBall->angle != 67)) {
			lpBall->angle = 225;
		} else if (b2 <= r3) {
			lpBall->angle = 247;
		} else if (b1 < r6) {
			lpBall->angle = 360 - lpBall->angle;
		} else {
			if (lpBall->angle == 67) lpBall->angle = 315;
			else lpBall->angle = 337;
		}
	} else {
		/*    'x' . */
		/*
		 *      
		 *    .
		 */
		if (b1 >= r6) {
			if (lpBall->angle == 113) lpBall->angle = 315;
			else lpBall->angle = 337;
		} else if ((b1 >= r5) && (lpBall->angle != 113)) {
			lpBall->angle = 315;
		} else if (b1 >= r4) {
			lpBall->angle = 293;
		} else if (b2 > r1) {
			lpBall->angle = 360 - lpBall->angle;
		} else {
			if (lpBall->angle == 113) lpBall->angle = 225;
			else lpBall->angle = 203;
		}
	}

	/*   'y' */
	lpBall->iy = RACKET_Y - wBall;
	/*    */
	SetSpeedBall(lpBall);
	/*   */
	play.wall = 1;

	return 1;
}

/*        90  */
static int CheckBlock90(LPBALL lpBall)
{
	BOUND bound1,bound2,bound3,bound4,bound5;
	int b1,b2,b3,b4,b5;

	/*      */
	b1 = GetBlock(&bound1,lpBall->ix,lpBall->iy+7);     /* - */
	b2 = GetBlock(&bound2,lpBall->ix+6,lpBall->iy+7);   /* - */
	b3 = GetBlock(&bound3,lpBall->ix+7,lpBall->iy);     /* - */
	b4 = GetBlock(&bound4,lpBall->ix+7,lpBall->iy+6);   /* - */
	b5 = GetBlock(&bound5,lpBall->ix+7,lpBall->iy+7);   /*  - */

	/*       */
	if ((b1 && b3) && (bound1.x != bound3.x) &&
		(bound1.y != bound3.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*    */
		MarkBlock(bound1.x,bound1.y);
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = lpBall->angle + 180;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b1 && b2) && (bound1.x != bound2.x)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/* ,       */
		if ((16 - bound1.offset_x) > (bound2.offset_x + 1)) {
			MarkBlock(bound1.x,bound1.y);
		} else {
			MarkBlock(bound2.x,bound2.y);
		}
		/*   */
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b3 && b4) && (bound3.y != bound4.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/* ,       */
		if ((8 - bound3.offset_y) > (bound4.offset_y + 1)) {
			MarkBlock(bound3.x,bound3.y);
		} else {
			MarkBlock(bound4.x,bound4.y);
		}
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		/*   */
		lpBall->angle = 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*     */
	if (b1) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		/*   */
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*     */
	if (b3) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		/*   */
		lpBall->angle = 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if (b5) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound5.x,bound5.y);
		/*     */
		if ((bound5.offset_x + 1) > (bound5.offset_y + 1)) {
			lpBall->iy = bound5.y * 8 + LOGIC_Y - 7;
			lpBall->angle = 360 - lpBall->angle;
		} else if ((bound5.offset_x + 1) < (bound5.offset_y + 1)) {
			lpBall->ix = bound5.x * 16 + LOGIC_X - 7;
			lpBall->angle = 180 - lpBall->angle;
		} else {
			/*        */
			if (lpBall->angle < 45) {
				lpBall->ix = bound5.x * 16 + LOGIC_X - 7;
				lpBall->angle = 180 - lpBall->angle;
			} else {
				lpBall->iy = bound5.y * 8 + LOGIC_Y - 7;
				lpBall->angle = 360 - lpBall->angle;
			}
		}
		/*   */
		SetSpeedBall(lpBall);

		return 1;
	}

	return 0;
}

static int CheckBlock180(LPBALL lpBall)
{
	BOUND bound1,bound2,bound3,bound4,bound5;
	int b1,b2,b3,b4,b5;

	/*      */
	b1 = GetBlock(&bound1,lpBall->ix+6,lpBall->iy+7);       /* - */
	b2 = GetBlock(&bound2,lpBall->ix,lpBall->iy+7);         /* - */
	b3 = GetBlock(&bound3,lpBall->ix-1,lpBall->iy);         /* - */
	b4 = GetBlock(&bound4,lpBall->ix-1,lpBall->iy+6);       /* - */
	b5 = GetBlock(&bound5,lpBall->ix-1,lpBall->iy+7);       /*  - */

	/*       */
	if ((b1 && b3) && (bound1.x != bound3.x) &&
		(bound1.y != bound3.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*    */
		MarkBlock(bound1.x,bound1.y);
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = lpBall->angle + 180;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b1 && b2) && (bound1.x != bound2.x)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((bound1.offset_x + 1) > (16 - bound2.offset_x)) {
			MarkBlock(bound1.x,bound1.y);
		} else {
			MarkBlock(bound2.x,bound2.y);
		}
		/*   */
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b3 && b4) && (bound3.y != bound4.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((8 - bound3.offset_y) > (bound4.offset_y + 1)) {
			MarkBlock(bound3.x,bound3.y);
		} else {
			MarkBlock(bound4.x,bound4.y);
		}
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		/*   */
		lpBall->angle = 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*     */
	if (b1) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		/*   */
		lpBall->iy = bound1.y * 8 + LOGIC_Y - 7;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*     */
	if (b3) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		/*   */
		lpBall->angle = 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if (b5) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound5.x,bound5.y);
		/*     */
		if ((16 - bound5.offset_x) > (bound5.offset_y + 1)) {
			lpBall->iy = bound5.y * 8 + LOGIC_Y - 7;
			lpBall->angle = 360 - lpBall->angle;
		} else if ((16 - bound5.offset_x) < (bound5.offset_y + 1)) {
			lpBall->ix = (bound5.x + 1) * 16 + LOGIC_X;
			lpBall->angle = 180 - lpBall->angle;
		} else {
			/*        */
			if (lpBall->angle > 135) {
				lpBall->ix = (bound5.x + 1) * 16 + LOGIC_X;
				lpBall->angle = 180 - lpBall->angle;
			} else {
				lpBall->iy = bound5.y * 8 + LOGIC_Y - 7;
				lpBall->angle = 360 - lpBall->angle;
			}
		}
		/*   */
		SetSpeedBall(lpBall);

		return 1;
	}

	return 0;
}

static int CheckBlock270(LPBALL lpBall)
{
	BOUND bound1,bound2,bound3,bound4,bound5;
	int b1,b2,b3,b4,b5;

	/*      */
	b1 = GetBlock(&bound1,lpBall->ix+6,lpBall->iy-1);      /* - */
	b2 = GetBlock(&bound2,lpBall->ix,lpBall->iy-1);        /* - */
	b3 = GetBlock(&bound3,lpBall->ix-1,lpBall->iy+6);      /* - */
	b4 = GetBlock(&bound4,lpBall->ix-1,lpBall->iy);        /* - */
	b5 = GetBlock(&bound5,lpBall->ix-1,lpBall->iy-1);      /*  - */

	/*       */
	if ((b1 && b3) && (bound1.x != bound3.x) &&
		(bound1.y != bound3.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = lpBall->angle - 180;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b1 && b2) && (bound1.x != bound2.x)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((bound1.offset_x + 1) > (16 - bound2.offset_x)) {
			MarkBlock(bound1.x,bound1.y);
		} else {
			MarkBlock(bound2.x,bound2.y);
		}
		/*   */
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b3 && b4) && (bound3.y != bound4.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((bound3.offset_y + 1) > (8 - bound4.offset_y)) {
			MarkBlock(bound3.x,bound3.y);
		} else {
			MarkBlock(bound4.x,bound4.y);
		}
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		/*   */
		lpBall->angle = 360 + 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*  -  */
	if (b1) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		/*   */
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*  -  */
	if (b3) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = (bound3.x + 1) * 16 + LOGIC_X;
		/*   */
		lpBall->angle = 360 + 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if (b5) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound5.x,bound5.y);
		/*     */
		if ((16 - bound5.offset_x) > (8 - bound5.offset_y)) {
			lpBall->iy = (bound5.y + 1) * 8 + LOGIC_Y;
			lpBall->angle = 360 - lpBall->angle;
		} else if ((16 - bound5.offset_x) < (8 - bound5.offset_y)) {
			lpBall->ix = (bound5.x + 1) * 16 + LOGIC_X;
			lpBall->angle = 360 + 180 - lpBall->angle;
		} else {
			/*        */
			if (lpBall->angle < 225) {
				lpBall->ix = (bound5.x + 1) * 16 + LOGIC_X;
				lpBall->angle = 360 + 180 - lpBall->angle;
			} else {
				lpBall->iy = (bound5.y + 1) * 8 + LOGIC_Y;
				lpBall->angle = 360 - lpBall->angle;
			}
		}
		/*   */
		SetSpeedBall(lpBall);

		return 1;
	}

	return 0;
}

static int CheckBlock360(LPBALL lpBall)
{
	BOUND bound1,bound2,bound3,bound4,bound5;
	int b1,b2,b3,b4,b5;

	/*      */
	b1 = GetBlock(&bound1,lpBall->ix,lpBall->iy-1);       /* - */
	b2 = GetBlock(&bound2,lpBall->ix+6,lpBall->iy-1);     /* - */
	b3 = GetBlock(&bound3,lpBall->ix+7,lpBall->iy+6);     /* - */
	b4 = GetBlock(&bound4,lpBall->ix+7,lpBall->iy);       /* - */
	b5 = GetBlock(&bound5,lpBall->ix+7,lpBall->iy-1);     /*  - */

	/*       */
	if ((b1 && b3) && (bound1.x != bound3.x) &&
		(bound1.y != bound3.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = lpBall->angle - 180;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b1 && b2) && (bound1.x != bound2.x)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((16 - bound1.offset_x) > (bound2.offset_x + 1)) {
			MarkBlock(bound1.x,bound1.y);
		} else {
			MarkBlock(bound2.x,bound2.y);
		}
		/*   */
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*    */
	if ((b3 && b4) && (bound3.y != bound4.y)) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*        */
		if ((bound3.offset_y + 1) > (8 - bound4.offset_y)) {
			MarkBlock(bound3.x,bound3.y);
		} else {
			MarkBlock(bound4.x,bound4.y);
		}
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		/*   */
		lpBall->angle = 360 + 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*  -  */
	if (b1) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound1.x,bound1.y);
		/*   */
		lpBall->iy = (bound1.y + 1) * 8 + LOGIC_Y;
		/*   */
		lpBall->angle = 360 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*  -  */
	if (b3) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound3.x,bound3.y);
		/*   */
		lpBall->ix = bound3.x * 16 + LOGIC_X - 7;
		/*   */
		lpBall->angle = 360 + 180 - lpBall->angle;
		SetSpeedBall(lpBall);

		return 1;
	}

	/*   -  */
	if (b5) {
		/*     ,    */
		UnlockSwerve(lpBall);
		/*   */
		MarkBlock(bound5.x,bound5.y);
		/*      */
		if ((bound5.offset_x + 1) > (8 - bound5.offset_y)) {
			lpBall->iy = (bound5.y + 1) * 8 + LOGIC_Y;
			lpBall->angle = 360 - lpBall->angle;
		} else if ((bound5.offset_x + 1) < (8 - bound5.offset_y)) {
			lpBall->ix = bound5.x * 16 + LOGIC_X - 7;
			lpBall->angle = 360 + 180 - lpBall->angle;
		} else {
			/*        */
			if (lpBall->angle > 315) {
				lpBall->ix = bound5.x * 16 + LOGIC_X - 7;
				lpBall->angle = 360 + 180 - lpBall->angle;
			} else {
				lpBall->iy = (bound5.y + 1) * 8 + LOGIC_Y;
				lpBall->angle = 360 - lpBall->angle;
			}
		}
		/*   */
		SetSpeedBall(lpBall);

		return 1;
	}

	return 0;
}

/*     */
static int CheckCollideBlock(LPBALL lpBall)
{
	int flags;

	/*      */
	if (lpBall->angle < 90)
		flags = CheckBlock90(lpBall);
	else if (lpBall->angle < 180)
		flags = CheckBlock180(lpBall);
	else if (lpBall->angle < 270)
		flags = CheckBlock270(lpBall);
	else
		flags = CheckBlock360(lpBall);

	return flags;
}

/*      */
static int CheckCollideAlien(LPBALL lpBall)
{
	/*    */
	static const int angle90[] = {113,135,157,203,225,247,293,315,337};
	static const int angle180[] = {203,225,247,293,315,337,23,45,67};
	static const int angle270[] = {293,315,337,23,45,67,113,135,157};
	static const int angle360[] = {23,45,67,113,135,157,203,225,247};
	const int *angle;
	int x,y;

	/*      */
	if (!alien.on)
		return 0;

	/*    */
	x = lpBall->ix + 3;
	y = lpBall->iy + 3;

	/*      */
	if ((x < alien.ix + 1) || (x > alien.ix + 22))
		return 0;
	if ((y < alien.iy + 1) || (y > alien.iy + 12))
		return 0;

	/*     ,    */
	UnlockSwerve(lpBall);
	/*      */
	if (lpBall->angle < 90)
		angle = angle90;
	else if (lpBall->angle < 180)
		angle = angle180;
	else if (lpBall->angle < 270)
		angle = angle270;
	else
		angle = angle360;

	/*    */
	lpBall->angle = angle[random(9)];
	SetSpeedBall(lpBall);

	/*    */
	CreateBang();

	/*   */
	play.bang = 1;

	return 1;
}

/*     */
static void MoveBall(void)
{
	int i;

	/*    */
	for (i = 0; i < BALL_COUNT; i++) {
		LPBALL lpBall = &ball[i];
		LPFULL_PICTURE lpPicture = &scene.pBall[i];

		/*     */
		if (!lpBall->on)
			continue;

		/*   */
		lpBall->x += lpBall->vx;
		lpBall->y += lpBall->vy;

		/*     */
		lpBall->ix = ftol(lpBall->x);
		lpBall->iy = ftol(lpBall->y);

		/*     */
		CheckCollideRacket(lpBall);
		/*     */
		CheckCollideBorder(lpBall);
		/*     */
		CheckCollideBlock(lpBall);
		/*      */
		CheckCollideSwerve(lpBall);
		/*      */
		CheckCollideAlien(lpBall);

		/*    */
		lpPicture->picture.X = lpBall->ix;
		lpPicture->picture.Y = lpBall->iy;
		/*    */
		lpPicture->shadow.X = lpBall->ix + 7;
		lpPicture->shadow.Y = lpBall->iy + 7;
	}
}

/*     'x'  'y'    */
static void SetSpeedAlien(void)
{
	/*      'SetSpeedBall'. */
	double rad;

	/*     */
	rad = alien.angle * PI / 180.0;
	/*     'x'  'y' */
	alien.vx = alien.v * cos(rad);
	alien.vy = alien.v * sin(rad);
}

/*       */
static void CreateAlien(void)
{
	/*      */
	ZeroMemory(&alien,sizeof(alien));
	/*    */
	alien.on = 1;
	/*   */
	if (random(2)) alien.x = LOGIC_X + 160;
	else alien.x = LOGIC_X + 55;
	/*    */
	alien.ix = ftol(alien.x);
	alien.iy = ftol(alien.y);
	/*   */
	alien.v = 0.0;
	SetSpeedAlien();
	/*     */
	scene.pAlien.type = 0;
	if (nLevel % 2) {
		scene.pAlien.show = SHOW_ALIEN_UFO;
		alien.animation.frames = lpAlienUfo;
	} else {
		scene.pAlien.show = SHOW_ALIEN_BIRD;
		alien.animation.frames = lpAlienBird;
	}
}

/*    */
static void NextFrameAlien(void)
{
	int point = alien.animation.point;

	/*     */
	if (++alien.animation.count < alien.animation.frames[point].count)
		return;

	/*   */
	point++;
	/*     */
	if (alien.animation.frames[point].number == -1)
		point = 0;

	/*    */
	scene.pAlien.type = alien.animation.frames[point].number;
	/*    */
	alien.animation.point = point;
	alien.animation.count = 0;
}

/*      */
#define STATE_RUN                   1    /*  */
#define STATE_DETOUR                2    /*   */
#define STATE_CHANGE_COURSE         3    /*   */
#define STATE_CHANGE_SPEED          4    /*   */

/*       */
#define V_VIOLATION               0.8

/*    'DETOUR' */
#define DETOUR_VIOLATION_TOP        1    /*    */
#define DETOUR_VIOLATION_LEFT       2    /*    */
#define DETOUR_VIOLATION_RIGHT      3    /*    */

#define DETOUR_BLOCK_RIGHT          4    /*     */
#define DETOUR_BLOCK_BOTTOM         5    /*     */
#define DETOUR_BLOCK_LEFT           6    /*     */
#define DETOUR_BLOCK_TOP            7    /*     */

/*      */
static struct {
	int subject;                     /*    */
	int message;                     /*  */
} info;

/*        */
static void CollideAlien(void)
{
	BOUND bound;
	int b1,b2,b3,b4;
	RECT r,a;

	/*      */
	if (alien.iy + 2 >= LOGIC_Y + LOGIC_HEIGHT) {
		/*    */
		CreateAlien();
		return;
	}

	/*    */
	r.left = racket.x;
	r.top = racket.y;
	r.right = racket.x + GetWidthRacket() - 1;
	r.bottom = racket.y + GetHeightRacket() - 1;

	/*     */
	a.left = alien.ix + 5;
	a.top = alien.iy + 3;
	a.right = alien.ix + 18;
	a.bottom = alien.iy + 10;

	/*         */
	if (((r.left >= a.left && r.left <= a.right) ||
		(r.right >= a.left && r.right <= a.right)) &&
		((r.top >= a.top && r.top <= a.bottom) ||
		(r.bottom >= a.top && r.bottom <= a.bottom))) {
		/*      */
		CreateBang();
		return;
	}

	/*         */
	if (((a.left >= r.left && a.left <= r.right) ||
		(a.right >= r.left && a.right <= r.right)) &&
		((a.top >= r.top && a.top <= r.bottom) ||
		(a.bottom >= r.top && a.bottom <= r.bottom))) {
		/*      */
		CreateBang();
		return;
	}

	/*      */
	if (alien.ix + 7 <= LOGIC_X) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_VIOLATION_LEFT;

		return;
	}

	/*      */
	if (alien.ix + 16 >= LOGIC_X + LOGIC_WIDTH) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_VIOLATION_RIGHT;

		return;
	}

	/*      */
	if (alien.iy + 6 <= LOGIC_Y) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_VIOLATION_TOP;

		return;
	}

	/*      */
	b1 = GetBlock(&bound,alien.ix+13,alien.iy+7);   /*    */
	b2 = GetBlock(&bound,alien.ix+12,alien.iy+8);   /*    */
	b3 = GetBlock(&bound,alien.ix+10,alien.iy+7);   /*    */
	b4 = GetBlock(&bound,alien.ix+12,alien.iy+5);   /*    */

	/*   */
	if (b1) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_BLOCK_RIGHT;
	} else if (b2) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_BLOCK_BOTTOM;
	} else if (b3) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_BLOCK_LEFT;
	} else if (b4) {
		/*    'DETOUR' */
		alien.automat.state = STATE_DETOUR;
		alien.automat.count = 0;
		/*    */
		info.subject = STATE_DETOUR;
		info.message = DETOUR_BLOCK_TOP;
	}
}

/*    */
static void StateDetourAlien(void)
{
	/*      */
	if (alien.automat.count > 0) {
		alien.automat.count--;
		return;
	}

	/* ,   */
	if (info.subject == STATE_DETOUR) {
		/*    */
		switch (info.message) {
			case DETOUR_VIOLATION_LEFT:
				alien.angle = 0;
				break;

			case DETOUR_VIOLATION_RIGHT:
				alien.angle = 180;
				break;

			case DETOUR_VIOLATION_TOP:
				alien.angle = 90;
				break;

			case DETOUR_BLOCK_RIGHT:
				alien.angle = 160 + random(41);
				break;

			case DETOUR_BLOCK_BOTTOM:
				alien.angle = 250 + random(41);
				break;

			case DETOUR_BLOCK_LEFT:
				alien.angle = 340 + random(41);
				if (alien.angle >= 360)
					alien.angle -= 360;
				break;

			case DETOUR_BLOCK_TOP:
				alien.angle = 70 + random(41);
				break;
		}
		/*   */
		alien.v = V_VIOLATION;
		SetSpeedAlien();
		/*      */
		alien.automat.count = 2;
		info.subject = 0;
		return;
	}

	/*   */
	alien.automat.state = STATE_CHANGE_COURSE;
	alien.automat.count = 0;
}

/*   */
static void StateRunAlien(void)
{
	int state;

	/*      */
	if (alien.automat.count > 0) {
		alien.automat.count--;
		return;
	}

	/*    */
	state = random(6);
	if (state == 0) {
		/*    */
		alien.automat.count = 20 + random(30);
	} else if (state <= 2) {
		/*      */
		alien.automat.state = STATE_CHANGE_COURSE;
	} else {
		/*      */
		alien.automat.state = STATE_CHANGE_SPEED;
	}
}

/*   */
static void StateChangeCourseAlien(void)
{
	/*    */
	if (info.subject != STATE_CHANGE_COURSE) {
		/*    */
		if (random(2))
			info.message = random(3) + 1;
		else
			info.message = -(random(3) + 1);
		/*    */
		alien.automat.count = 30 + random(30);
		/*    */
		info.subject = STATE_CHANGE_COURSE;
	}

	/*     */
	if (alien.automat.count <= 0) {
		/*     */
		alien.automat.state = STATE_RUN;
		info.subject = 0;
		/*  */
		return;
	}

	/*    */
	alien.automat.count--;
	/*   */
	alien.angle += info.message;
	if (alien.angle < 0)
		alien.angle += 360;
	else if (alien.angle >= 360)
		alien.angle -= 360;
	SetSpeedAlien();
}

/*   */
static void StateChangeSpeedAlien(void)
{
	/*   */
	if (random(2)) alien.v += 0.2;
	else alien.v -= 0.2;
	/*   */
	if (alien.v < 0.2) {
		alien.v = 0.2;
	} else if (alien.v > 1.0) {
		alien.v = 1.0;
	}

	/*     */
	alien.automat.state = STATE_RUN;
	alien.automat.count = 10 + random(20);
	/*   */
	SetSpeedAlien();
}

/*    */
static void MoveAlien(void)
{
	/*      */
	if (!alien.on) {
		/*      */
		int level = nLevel % LEVEL_COUNT;
		int AlienBlock = levels[level].AlienBlock;

		/*     */
		if (!AlienBlock || bang.on)
			return;

		/*      */
		if ((AlienBlock != -1) && (AlienBlock < GetCountBlock()))
			return;

		/*    */
		CreateAlien();
	}

	/*    */
	alien.x += alien.vx;
	alien.y += alien.vy;
	/*    */
	alien.ix = ftol(alien.x);
	alien.iy = ftol(alien.y);

	/*   ,    '' */
	if (alien.automat.state != STATE_DETOUR) {
		CollideAlien();
		/* ,      */
		if (!alien.on)
			return;
	}

	/*       */
	switch (alien.automat.state) {
		/*  */
		case STATE_RUN:
			StateRunAlien();
			break;
		/*   */
		case STATE_DETOUR:
			StateDetourAlien();
			break;
		/*   */
		case STATE_CHANGE_COURSE:
			StateChangeCourseAlien();
			break;
		/*   */
		case STATE_CHANGE_SPEED:
			StateChangeSpeedAlien();
			break;
		/*     */
		default:
			alien.automat.state = STATE_RUN;
			alien.automat.count = 0;
	}

	/*     */
	NextFrameAlien();
	/*   */
	scene.pAlien.X = alien.ix;
	scene.pAlien.Y = alien.iy;
}

/*   */
static void TransferBomb(void)
{
	RECT r;
	POINT p;

	/*   */
	bomb.y += bomb.vy;
	/*   */
	if (bomb.vy < SPEED_Y) {
		/*   */
		bomb.vy += ACCELERATION_Y;
		if (bomb.vy > SPEED_Y)
			bomb.vy = SPEED_Y;
	}
	/*    'y' */
	bomb.iy = ftol(bomb.y);

	/*    */
	r.left = racket.x;
	r.top = racket.y;
	r.right = racket.x + GetWidthRacket() - 1;
	r.bottom = racket.y + GetHeightRacket() - 1;

	/*     */
	p.x = bomb.ix + 3;
	p.y = bomb.iy + 5;

	/*     */
	if ((p.x >= r.left && p.x <= r.right) &&
		(p.y >= r.top && p.y <= r.bottom)) {
		/*   */
		DestroyRacket();
		return;
	}

	/*     */
	if (bomb.iy > (LOGIC_Y + LOGIC_HEIGHT)) {
		/*     */
		bomb.on = 0;
		scene.pBomb.picture.show = 0;
		scene.pBomb.shadow.show = 0;
		/*    */
		bomb.count = 25;
	} else {
		/*
		 *      
		 * ( 'x'  ,  ).
		 */
		scene.pBomb.picture.Y = bomb.iy;
		scene.pBomb.shadow.Y = bomb.iy + 7;
	}
}

/*   */
static void CreateBomb(void)
{
	/*    */
	if (bomb.count > 0) {
		bomb.count--;
		return;
	}
	/*    */
	if (random(3)) {
		bomb.count = 10 + random(20);
		return;
	}

	/*   */
	bomb.on = 1;
	bomb.ix = alien.ix + 8;
	bomb.iy = alien.iy + 4;
	/*    */
	bomb.y = bomb.iy;
	bomb.vy = 0.0;

	/*     */
	scene.pBomb.picture.show = 1;
	scene.pBomb.picture.X = bomb.ix;
	scene.pBomb.picture.Y = bomb.iy;
	scene.pBomb.shadow.show = 1;
	scene.pBomb.shadow.X = bomb.ix + 8;
	scene.pBomb.shadow.Y = bomb.iy + 7;
}

/*   */
static void MoveBomb(void)
{
	/*   ,    */
	if (bomb.on)
		TransferBomb();
	/*       */
	else if (alien.on)
		CreateBomb();
}

/*    */
static void RunBonusPoints(void)
{
}

/*   "Kill Aliens" */
static void RunBonusSmartBomb(void)
{
}

/*    */
static void RunBonusExtraLife(void)
{
}

/*   " " */
static void RunBonusSlowBall(void)
{
}

/*   " " */
static void RunBonusSmashBall(void)
{
}

/*   " " */
static void RunBonusTripleBall(void)
{
	FULL_PICTURE pBall;
	BALL lBall;
	int angle,angle1,angle2;
	int i;

	/*    */
	for (i = 0; i < BALL_COUNT; i++) {
		if (ball[i].on) {
			/*    */
			CopyMemory(&pBall,&scene.pBall[i],sizeof(pBall));
			CopyMemory(&lBall,&ball[i],sizeof(lBall));
			break;
		}
	}

	/* ,    */
	if (i == BALL_COUNT)
		return;

	/*     */
	if (lBall.swerve.lock)
		angle = lBall.swerve.angle;
	else
		angle = lBall.angle;

	/*    */
	if (angle < 90) {
		switch (angle) {
			case 23: angle1 = 45; angle2 = 67; break;
			case 45: angle1 = 23; angle2 = 67; break;
			case 67: angle1 = 23; angle2 = 45; break;
			default: return;
		}
	} else if (angle < 180) {
		switch (angle) {
			case 113: angle1 = 135; angle2 = 157; break;
			case 135: angle1 = 113; angle2 = 157; break;
			case 157: angle1 = 113; angle2 = 135; break;
			default: return;
		}
	} else if (angle < 270) {
		switch (angle) {
			case 203: angle1 = 225; angle2 = 247; break;
			case 225: angle1 = 203; angle2 = 247; break;
			case 247: angle1 = 203; angle2 = 225; break;
			default: return;
		}
	} else {
		switch (angle) {
			case 293: angle1 = 315; angle2 = 337; break;
			case 315: angle1 = 293; angle2 = 337; break;
			case 337: angle1 = 293; angle2 = 315; break;
			default: return;
		}
	}

	/*   */
	CopyMemory(&scene.pBall[0],&pBall,sizeof(pBall));
	CopyMemory(&ball[0],&lBall,sizeof(lBall));
	SetSpeedBall(&ball[0]);

	CopyMemory(&scene.pBall[1],&pBall,sizeof(pBall));
	CopyMemory(&ball[1],&lBall,sizeof(lBall));
	ball[1].angle = angle1;
	ball[1].swerve.lock = 0;
	SetSpeedBall(&ball[1]);

	CopyMemory(&scene.pBall[2],&pBall,sizeof(pBall));
	CopyMemory(&ball[2],&lBall,sizeof(lBall));
	ball[2].angle = angle2;
	ball[2].swerve.lock = 0;
	SetSpeedBall(&ball[2]);
}

/*   "" */
static void RunBonusHand(void)
{
}

/*   " " */
static void RunBonusGun(void)
{
}

/*   " " */
static void RunBonusExtendedRacket(void)
{
}

/*   "" */
static void RunBonusRocketPack(void)
{
}

/*    */
static void RunBonus(void)
{
	switch (scene.pBonus.type) {
		case BONUS_POINTS:
			RunBonusPoints();
			break;
		case BONUS_SMART_BOMB:
			RunBonusSmartBomb();
			break;
		case BONUS_EXTRA_LIFE:
			RunBonusExtraLife();
			break;
		case BONUS_SLOW_BALL:
			RunBonusSlowBall();
			break;
		case BONUS_SMASH_BALL:
			RunBonusSmashBall();
			break;
		case BONUS_TRIPLE_BALL:
			RunBonusTripleBall();
			break;
		case BONUS_HAND:
			RunBonusHand();
			break;
		case BONUS_GUN:
			RunBonusGun();
			break;
		case BONUS_EXTENDED_RACKET:
			RunBonusExtendedRacket();
			break;
		case BONUS_ROCKET_PACK:
			RunBonusRocketPack();
			break;
	}
}

/*      */
static void CreateBonus400Points(void)
{
	int angle,r,b;
	double rad,speed;

	/*     */
	r = racket.x + GetWidthRacket() / 2;
	b = bonus.ix + GetWidthBonus() / 2;

	/*        */
	if (racket.type != TYPE_RACKET_EXTEND) {
		if (b < (r - 9)) {
			angle = 198 + random(10);
			speed = 1.05;
		} else if (b < (r - 6)) {
			angle = 220 + random(10);
			speed = 0.95;
		} else if (b < r) {
			angle = 242 + random(10);
			speed = 0.85;
		} else if (b <= (r + 6)) {
			angle = 288 + random(10);
			speed = 0.85;
		} else if (b <= (r + 9)) {
			angle = 310 + random(10);
			speed = 0.95;
		} else {
			angle = 332 + random(10);
			speed = 1.05;
		}
	} else {
		if (b < (r - 15)) {
			angle = 198 + random(10);
			speed = 1.05;
		} else if (b < (r - 10)) {
			angle = 220 + random(10);
			speed = 0.95;
		} else if (b < r) {
			angle = 242 + random(10);
			speed = 0.85;
		} else if (b <= (r + 10)) {
			angle = 288 + random(10);
			speed = 0.85;
		} else if (b <= (r + 15)) {
			angle = 310 + random(10);
			speed = 0.95;
		} else {
			angle = 332 + random(10);
			speed = 1.05;
		}
	}

	/*     */
	rad = angle * PI / 180.0;
	/*     */
	bonus.vx = speed * cos(rad);
	bonus.vy = speed * sin(rad);

	/*    */
	RunBonus();

	/*     */
	scene.pBonus.type = BONUS_400_POINTS;
}

/*    */
static void MoveBonus(void)
{
	/*       */
	if (!bonus.on)
		return;

	/*    */
	if (scene.pBonus.type != BONUS_400_POINTS) {
		RECT r,b;
		/*   */
		bonus.y += bonus.vy;
		/*   */
		if (bonus.vy < SPEED_Y) {
			/*   */
			bonus.vy += ACCELERATION_Y;
			if (bonus.vy > SPEED_Y)
				bonus.vy = SPEED_Y;
		}
		/*     */
		bonus.iy = ftol(bonus.y);
		/*    */
		r.left = racket.x;
		r.top = racket.y;
		r.right = racket.x + GetWidthRacket() - 1;
		r.bottom = racket.y + GetHeightRacket() - 1;
		/*    */
		b.left = bonus.ix + 4;
		b.top = bonus.iy + 4;
		b.right = bonus.ix + GetWidthBonus() - 1 - 4;
		b.bottom = bonus.iy + GetHeightBonus() - 1 - 4;
		/*        */
		if (((r.left >= b.left && r.left <= b.right) ||
			(r.right >= b.left && r.right <= b.right)) &&
			((r.top >= b.top && r.top <= b.bottom) ||
			(r.bottom >= b.top && r.bottom <= b.bottom))) {
			/*     */
			CreateBonus400Points();
		} else if (((b.left >= r.left && b.left <= r.right) ||
			(b.right >= r.left && b.right <= r.right)) &&
			((b.top >= r.top && b.top <= r.bottom) ||
			(b.bottom >= r.top && b.bottom <= r.bottom))) {
			/*     */
			CreateBonus400Points();
		}
		/*    */
		scene.pBonus.Y = bonus.iy;
		scene.pBonus.X = bonus.ix;
	} else {
		/*    */
		int width = GetWidthBonus();
		/*   */
		bonus.y += bonus.vy;
		bonus.x += bonus.vx;
		/*   */
		bonus.vy += 0.04;
		/*     */
		bonus.iy = ftol(bonus.y);
		bonus.ix = ftol(bonus.x);
		/*     */
		if ((bonus.ix <= LOGIC_X) && (bonus.vx < 0)) {
			bonus.vx = -bonus.vx;
			bonus.ix = LOGIC_X;
		} else if ((bonus.ix >= (LOGIC_X + LOGIC_WIDTH - width)) && (bonus.vx > 0)) {
			bonus.vx = -bonus.vx;
			bonus.ix = LOGIC_X + LOGIC_WIDTH - width;
		}
		/*    */
		scene.pBonus.Y = bonus.iy;
		scene.pBonus.X = bonus.ix;
	}

	/*       */
	if (bonus.iy > (LOGIC_Y + LOGIC_HEIGHT)) {
		/*   */
		bonus.on = 0;
		scene.pBonus.show = 0;
	}
}

/*    */
static void MoveSwerves(void)
{
	int i;

	/*     */
	for (i = 0; i < SWERVES_COUNT; i++) {
		/*   */
		LPPICTURE lpPicture = &scene.pSwerves[i];
		LPSWERVE lpSwerve = &swerve[i];

		/*   ,   */
		if (lpPicture->show) {
			/*     */
			if (--lpSwerve->count > 0)
				continue;
			/*   */
			if (random(100) < 45) {
				if (lpSwerve->on) {
					/* 45%  */
					lpSwerve->on = 0;
					lpPicture->type = PICTURE_SWERVE_OFF;
					/*   */
					play.swerve = 1;
				}
			} else {
				if (!lpSwerve->on) {
					/* 55%  */
					lpSwerve->on = 1;
					lpPicture->type = PICTURE_SWERVE_ON;
					/*   */
					play.swerve = 1;
				}
			}
			/*     */
			lpSwerve->count = 30 + random(100);
		}
	}
}

/*      */
static void MoveBang(void)
{
	/*    */
	if (!bang.on)
		return;

	/*     */
	if (++bang.count < lpBang[bang.point].count)
		return;

	/*   */
	bang.point++;
	bang.count = 0;
	/*    */
	if (lpBang[bang.point].number == -1) {
		bang.on = 0;
		scene.pBang.show = 0;
		return;
	}

	/*    */
	scene.pBang.type = lpBang[bang.point].number;
}

/*    */
static int MoveAnima(void)
{
	/*    ,   */
	return (anima.func) ? (*anima.func)() : 0;
}

/*     */
int MoveScene(void)
{
	static int count;

	/*    */
	CheckControlKey();

	if (pause.pause)
		return 0;

	/*  */
	MoveRacket();

	if (slip) {
		if (++count % 20)
			return 0;
	}

	/*   */
	switch (MoveAnima()) {
		/*    */
		case 0: break;
		/*      */
		case 1: goto COPY_BLOCK;
		/*    */
		default: return 0;
	}

	/*   */
//	MoveRacket();
	/*    */
	MoveSwerves();
	/*    */
	MoveAlien();
	/*   */
	MoveBomb();
	/*    */
	MoveBonus();
	/*     */
	MoveBang();

	/*      */
	if (hand) {
		int i;
		for (i = 0; i < BALL_COUNT; i++) {
			if (scene.pBall[i].picture.show) {
				ball[i].x = scene.pBall[i].picture.X = racket.x + 12;
				ball[i].y = scene.pBall[i].picture.Y = racket.y - 6;
				scene.pBall[i].shadow.X = scene.pBall[i].picture.X + 7;
				scene.pBall[i].shadow.Y = scene.pBall[i].picture.Y + 7;
			}
		}
	} else {
		MoveBall();
	}

COPY_BLOCK:
	/*     */
	CopyBlock();

	return 0;
}

/*   */
int CreateLevel(int level)
{
	/*    */
	nLevel = level;

	/*   */
	ZeroMemory(&scene,sizeof(scene));

	/*     */
	level = level % LEVEL_COUNT;

	/*    */
	scene.on = 1;
	scene.background = level % 4 + BACKGROUND_0;

	/*    */
	ZeroMemory(block,sizeof(block));
	CopyMemory(&block[3][0],levels[level].round,sizeof(UCHAR)*(15*12));

	/*       */
	SetRacket();
	/*      */
	SetBall();
	/*    */
	SetSwerve();

	/*    */
	ZeroMemory(EventBlock,sizeof(EventBlock));
	ZeroMemory(&alien,sizeof(alien));
	ZeroMemory(&bang,sizeof(bang));
	ZeroMemory(&bomb,sizeof(bomb));
	ZeroMemory(&bonus,sizeof(bonus));
	ZeroMemory(&anima,sizeof(anima));
	ZeroMemory(sliver,sizeof(sliver));
	/*       */
	
	/*     */
	CopyBlock();

	hand = 1;

	return 0;
}
