/* main.c */
#include "global.h"
#include <stdlib.h>
#include "game.h"

/*    */
static const TCHAR szWindowClassName[] = _T("WINCLASS1");
static const TCHAR szWindowTitle[] = _T("Batty");

/*   */
HINSTANCE ghInst;                  /*   */
HWND ghWndMain;                    /*   */

/*    FPS */
int bFps;

/*   */
static LRESULT CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;

	switch (msg) {
		case WM_CREATE:
			break;

		case WM_PAINT:
			hdc = BeginPaint(hWnd,&ps);
			EndPaint(hWnd,&ps);
			break;

		case WM_DESTROY:
			PostQuitMessage(0);
			break;

		default:
			return DefWindowProc(hWnd,msg,wParam,lParam);
	}

	return 0;
}

/*     */
static int VersionCheck(void)
{
	OSVERSIONINFO os;

	/*      */
	ZeroMemory(&os,sizeof(os));
	os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	if (!GetVersionEx(&os))
		return 1;

	/*    */
	return (os.dwPlatformId == VER_PLATFORM_WIN32_NT) ? 0 : 1;
}

/*       */
static int InitializeApp(HINSTANCE hInst, int nCmdShow)
{
	WNDCLASSEX wc;

	/*    */
	ghInst = hInst;

	/*     */
	ZeroMemory(&wc,sizeof(wc));
	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	wc.lpfnWndProc = WindowProc;
	wc.hInstance = hInst;
	wc.hIcon = LoadIcon(NULL,IDI_APPLICATION);
	wc.hIconSm = LoadIcon(NULL,IDI_APPLICATION);
	wc.hCursor = LoadCursor(NULL,IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszClassName = szWindowClassName;

	/*    */
	if (!RegisterClassEx(&wc))
		return 1;

	/*   */
	ghWndMain = CreateWindowEx(0,szWindowClassName,szWindowTitle,WS_POPUP,
		0,0,SCREEN_WIDTH,SCREEN_HEIGHT,NULL,NULL,hInst,NULL);
	if (!ghWndMain)
		return 1;

	/*   */
	ShowWindow(ghWndMain,nCmdShow);
	UpdateWindow(ghWndMain);

	return 0;
}

/*     */
int WINAPI _tWinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow)
{
	MSG msg;

	/*     */
	if (VersionCheck()) {
		MessageBox(NULL,_T("Error check Windows."),
			_T("Error"),MB_OK|MB_ICONSTOP);
		return 1;
	}

	/*     */
	if (InitializeApp(hInst,nCmdShow)) {
		MessageBox(NULL,_T("Error start game."),
			_T("Error"),MB_OK|MB_ICONSTOP);
		return 1;
	}

	/*   FPS */
	bFps = 1;

	/*   */
	if (InitGame()) {
		ShutdownGame();
		ShowWindow(ghWndMain,SW_HIDE);
		MessageBox(ghWndMain,_T("Error run game."),
			_T("Error"),MB_OK|MB_ICONSTOP);
		return 1;
	}

	/*    */
	for(;;) {
		/*   ,    */
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			/*     */
			if (msg.message == WM_QUIT)
				break;
			/*     */
			TranslateMessage(&msg);
			DispatchMessage(&msg);
			/*    */
			continue;
		}

		/*    */
		MainGame();
	}

	/*   */
	ShutdownGame();

	return msg.wParam;
}
